package ru.mai.dep806.collections;

import java.util.*;

/**
 * Created by EUGENEL on 05.03.2015.
 */
public class MapSample {
    public static void main(String[] args) {

        Map<String, Integer> phoneBook =
                Collections.checkedMap(
                Collections.unmodifiableMap(
                    Collections.synchronizedMap(new HashMap<String, Integer>())
                ),
                String.class, Integer.class);

        phoneBook.put("John Smith", 1112233);
        System.out.println(phoneBook.get("John Smith"));


        final Map<Person, Integer> phoneBook2 = Collections.unmodifiableMap(new LinkedHashMap<Person, Integer>());

        phoneBook2.put(new Person("John", "Smith"), 2223344);

        System.out.println(phoneBook2.get(new Person("john", "smith")));

        Set<Person> personSet = Collections.checkedSet(new HashSet<Person>(), Person.class);
        personSet.add(new Person("Alex", "Brown"));
        personSet.add(new Person("alex", "brown"));
        System.out.println(personSet.size());


        Map<Person, Integer> phoneBook3 = new TreeMap<Person, Integer>(new MyNameComparator(MyNameComparator.SortPolicy.SORT_BY_FIRSTNAME));

        phoneBook3.put(new Person("Simon", "Taylor"), 4445566);
        phoneBook3.put(new Person("Alex", "Smith"), 4445567);
        phoneBook3.put(new Person("Charles", "Smith"), 4445567);
        phoneBook3.put(new Person("Steve", "Brown"), 4445568);
        System.out.println(phoneBook3.get(new Person("simon", "taylor")));
        for (Person p : phoneBook3.keySet()) {
            System.out.println(p);
        }

    }

    public static class Person implements Comparable<Person> {
        String firstName;
        String lastName;

        public Person(String firstName, String lastName) {
            this.firstName = firstName;
            this.lastName = lastName;
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            Person person = (Person) o;

            if (firstName != null ? !firstName.equalsIgnoreCase(person.firstName) : person.firstName != null) return false;
            if (lastName != null ? !lastName.equalsIgnoreCase(person.lastName) : person.lastName != null) return false;

            return true;
        }

        @Override
        public int hashCode() {
            int result = firstName != null ? firstName.toUpperCase().hashCode() : 0;
            result = 31 * result + (lastName != null ? lastName.toUpperCase().hashCode() : 0);
            return result;
        }

        @Override
        public int compareTo(Person o) {
            if (this.equals(o)) {
                return 0;
            }
            String fullName = lastName + firstName;
            String otherFullName = o.lastName + o.firstName;
            return fullName.toLowerCase().compareTo(otherFullName.toLowerCase());
        }

        @Override
        public String toString() {
            return "Person{" +
                    "firstName='" + firstName + '\'' +
                    ", lastName='" + lastName + '\'' +
                    '}';
        }
    }
}
